import java.util.*;
import java.io.*;

public class Main {
  static char POPTAG=129;
  static char SKIPPOPTAG=255;
  static char ZERO=200; // ONE = -101, -102...
  static char NINE=209;
  static char TYPEWRITER=130;
  static char PLAIN=131;
  static char BOLD=132;
  static char ITALIC=133;
  static char STRONG=134;
  static char EM=135;
  static char UNDERLINE=136;
  // colours...
  static char RED=180;
  static char GREEN=181;
  static char BLUE=182;
  static char CYAN=183;
  static char MAGENTA=184;
  static char YELLOW=185;
  static char BLACK=186;
  static char WHITE=187;

  char mBuffer[];
  int mCount;
  Hashtable mTagHash = new Hashtable();

  Main() {
    mBuffer = new char[6000000];
    mCount = 0;
    mTagHash.put(new Integer(RED),"r");
    mTagHash.put(new Integer(GREEN),"g");
    mTagHash.put(new Integer(BLUE),"b");
    mTagHash.put(new Integer(CYAN),"c");
    mTagHash.put(new Integer(MAGENTA),"m");
    mTagHash.put(new Integer(YELLOW),"y");
    mTagHash.put(new Integer(BLACK),"k");
    mTagHash.put(new Integer(WHITE),"w");
    for(int c=0;c<=9;++c) {
      mTagHash.put(new Integer(ZERO+c),Integer.toString(c));
    }
    mTagHash.put(new Integer(TYPEWRITER),"TT");
    mTagHash.put(new Integer(PLAIN),"PL");
    mTagHash.put(new Integer(BOLD),"B");
    mTagHash.put(new Integer(ITALIC),"I");
    mTagHash.put(new Integer(STRONG),"S");
    mTagHash.put(new Integer(EM),"EM");
    mTagHash.put(new Integer(UNDERLINE),"U");
  }

  public void outputBuffer() {
   try {
    Vector tagStack = new Vector();
    Integer i = new Integer(0);
    char ch;
    OutputStream os = new BufferedOutputStream(System.out);
       
    for(int c=0;c<mCount;++c) {
      ch = mBuffer[c];
      if (ch < 128) {
        os.write(ch);
      } else {
        // it had better be a tag...
        if (ch == POPTAG) {
          i = (Integer)tagStack.lastElement();
          tagStack.removeElementAt(tagStack.size()-1);
          if (i.intValue() != SKIPPOPTAG) {
            os.write(("</" + mTagHash.get(i) + ">").getBytes());
          }
        } else {
          i = new Integer(ch);
          if (ch != SKIPPOPTAG) {
            os.write(("<" + mTagHash.get(i) + ">").getBytes());
          }
          tagStack.addElement(i);
        }
      }
    }
    os.close();
   } catch (Exception e) {
    e.printStackTrace();
   }
  }

  //
  // This function will perform our space concatenation...
  // And will remove unnecessary <TAG></TAG> entries...
  // And will remove EM's in S's... By checking lack of change in context...
  //   so this also catches <b><b>Whatever</b></b>
  //
  public void inputBuffer() {
   Vector contextStack = new Vector();

   Context context = new Context();

   try {
    InputStream is = new BufferedInputStream(System.in);

    mCount = 0; // offset into buffer...

    int ch;
    while((ch = is.read()) != -1) {
      if (ch == '<') {
        // A tag...
        int nextCh = is.read();
        if (nextCh == '/') {
          int nextNextCh = is.read();
          if (mBuffer[mCount-1] >= 128 && mBuffer[mCount-1] != POPTAG) {
            // we're popping off a tag we just put on...
            mCount--;
          } else {
            mBuffer[mCount++] = POPTAG;
            // closing a tag - POP...
            // we don't need the read... keep reading...
          }
          while((ch = is.read()) != '>'); // we can do this because do is valid.

          // Pop out last context off the stack...
	  context = (Context)contextStack.lastElement();
          contextStack.removeElementAt(contextStack.size()-1);
          // System.out.println("After Pop: " + contextStack);
        } else {
          char tag;
          if (nextCh >= '0' && nextCh <= '9') {
            tag = (char)(ZERO + (nextCh-'0'));
            mBuffer[mCount++] = tag;
          } else if (nextCh == 'B') {
	    tag = BOLD;
            mBuffer[mCount++] = tag;
          } else if (nextCh == 'I') {
	    tag = ITALIC;
            mBuffer[mCount++] = tag;
          } else if (nextCh == 'S') {
	    tag = STRONG;
            mBuffer[mCount++] = tag;
          } else if (nextCh == 'U') {
	    tag = UNDERLINE;
            mBuffer[mCount++] = tag;
          } else if (nextCh == 'r') {
	    tag = RED;
            mBuffer[mCount++] = tag;
          } else if (nextCh == 'g') {
	    tag = GREEN;
            mBuffer[mCount++] = tag;
          } else if (nextCh == 'b') {
	    tag = BLUE;
            mBuffer[mCount++] = tag;
          } else if (nextCh == 'c') {
	    tag = CYAN;
            mBuffer[mCount++] = tag;
          } else if (nextCh == 'm') {
	    tag = MAGENTA;
            mBuffer[mCount++] = tag;
          } else if (nextCh == 'y') {
	    tag = YELLOW;
            mBuffer[mCount++] = tag;
          } else if (nextCh == 'k') {
	    tag = BLACK;
            mBuffer[mCount++] = tag;
          } else if (nextCh == 'w') {
	    tag = WHITE;
            mBuffer[mCount++] = tag;
          } else if (nextCh == 'E') {
            // Must be EM
            // Only add if we're not in S...
            tag = EM;
            mBuffer[mCount++] = tag;
            is.read();
          } else if (nextCh == 'P') {
	    tag = PLAIN;
            mBuffer[mCount++] = tag;
            is.read();
          } else if (nextCh == 'T') {
	    tag = TYPEWRITER;
            mBuffer[mCount++] = tag;
            is.read();
          } else {
            // TODO: Remove...
            tag = PLAIN; // NO NOT REALLY...
            System.err.println("Unknown nextCh: " + nextCh);
          }
          is.read();
          // Push our current context onto the stack...
          Context oldContext = context;
          contextStack.addElement(oldContext);
// System.out.println("After push: " + contextStack);
          // copy and update our context...
          context = new Context(context);
          context.setTag(tag);
          if (context.equals(oldContext)) {
            // Tag added did not affect context... Let's throw it and it's
            // buddy out...
            mBuffer[mCount-1] = SKIPPOPTAG;
          }
        }
      } else if (ch == ' ' || ch == 10 || ch == 13 || ch == 9) {
        // whitespace...
        if (mCount == 0) { 
          mBuffer[mCount++] = ' ';
        } else if (context.getTTFlag()) {
          mBuffer[mCount++] = (char)ch;
        } else if (mBuffer[mCount-1] == ' ' && !context.getTTFlag()) {
          ; 
        } else {
          mBuffer[mCount++] = ' '; 
        }
      } else {
        // must be a printable character...
        mBuffer[mCount++] = (char)ch;
// System.out.println("Printable: " + (char)ch + " " + context);
      }
    }
    // finished reading...
   } catch (Exception e) {
    e.printStackTrace();
   }


  }

  public static void main(String argv[]) {
    Main m = new Main();
    m.inputBuffer();
    m.outputBuffer();
  }

}
